@@ -41,30 +41,77 @@ class MyTextWrapper(textwrap.TextWrapper):
         self.wordsep_re_uni = re.compile(self.wordsep_re.pattern, re.U)
 
         # Override end of line regex, double space after '].'
-        self.sentence_end_re = re.compile(r'[%s%s0-9\>\]\)\"\']'  # lowercase letter, end parentheses
+        self.sentence_end_re = re.compile(r'[%s%s0-9\>\]\)\"\']'  # letter, end parentheses
                             r'[\.\!\?]'     # sentence-ending punct.
                             r'[\"\'\)\]]?'  # optional end-of-quote, end parentheses
                             r'\Z'           # end of chunk
                             % (string.lowercase, string.uppercase))
 
+        # Exceptions which should not be treated like end-of-sentence
+        self.not_sentence_end_re = re.compile(
+            # start of string or non-alpha character
+            r'(^|[^%s%s])'
+            r'('
+            # Single uppercase letter, dot, enclosing parentheses or quotes
+            r'[A-Za-z]\.[\]\)\'"]*'
+            # Tla with leading uppercase, and special cases
+            r'|([A-Z][a-z][a-z]|Eq|[Cc]f|vs|resp|viz|ibid|[JS]r|M[rs]|Messrs|Mmes|Dr|Profs?|St|Lt)\.'
+            r')\Z' # trailing dot, end of group and end of chunk
+            %(string.lowercase, string.uppercase) )
+
+        # Start of next sentence regex
+        self.sentence_start_re = re.compile("^[\"'([]*[%s]" % string.uppercase)
+
+        # XmlCharRef replacements that occur AFTER line breaking logic
+        self.post_break_replacements = {
+            '&#160;': ' ',   # nbsp
+            '&#8209;': '-',  # nbhy
+            '&#8288;': '',   # wj
+        }
+
+        self.break_on_hyphens = True
+
+
+    def _fix_sentence_endings(self, chunks):
+        """_fix_sentence_endings(chunks : [string])
+
+        Correct for sentence endings buried in 'chunks'.  Eg. when the
+        original text contains "... foo.\nBar ...", munge_whitespace()
+        and split() will convert that to [..., "foo.", " ", "Bar", ...]
+        which has one too few spaces; this method simply changes the one
+        space to two.
+        """
+        i = 0
+        patsearch = self.sentence_end_re.search
+        skipsearch = self.not_sentence_end_re.search
+        startsearch = self.sentence_start_re.search
+        while i < len(chunks)-2:
+            if (chunks[i+1] == " " and patsearch(chunks[i])
+                                  and skipsearch(chunks[i])==None
+                                  and startsearch(chunks[i+2])):
+                chunks[i+1] = "  "
+                i += 2
+            else:
+                i += 1
+
     def replace(self, text):
         """ Replace control entities with the proper character 
             after breaking has occured.
         """
-        for key, val in _post_break_replacements.items():
+        for key, val in self.post_break_replacements.items():
             text = re.sub(re.escape(key), val, text)
         return text
 
-    def wrap(self, text, break_on_hyphens=True, initial_indent='', subsequent_indent='',
-        fix_doublespace=True, fix_sentence_endings=True):
+    def wrap(self, text, initial_indent='', subsequent_indent='',
+        fix_doublespace=True, fix_sentence_endings=True, drop_whitespace=True):
         """ Mirrored implementation of wrap which replaces characters properly
             also lets you easily specify indentation on the fly
         """
         # Set indentation
         self.initial_indent = initial_indent
         self.subsequent_indent = subsequent_indent
-        self.break_on_hyphens = break_on_hyphens
         self.fix_sentence_endings = fix_sentence_endings
+        self.drop_whitespace = drop_whitespace
 
         # Original implementation
         text = self._munge_whitespace(text)
@@ -114,7 +161,10 @@ def justify_inline(left_str, center_str, right_str, width=72):
                          'too long:\n  ' + strings[longest_index])
         strings[longest_index] = strings[longest_index][:-(sumwidth - width)]
 
-    center = strings[1].center(width)
+    if len(strings[1]) % 2 == 0:
+        center = strings[1].center(width)
+    else:
+        center = strings[1].center(width+1)        
     right = strings[2].rjust(width)
     output = list(strings[0].ljust(width))
     for i, char in enumerate(output):
@@ -417,20 +467,3 @@ _unicode_replacements = {
     u'\u017e': 'z',
     u'\u2010': '-',
 }
-
-# XmlCharRef replacements that occur AFTER line breaking logic
-_post_break_replacements = {
-    '&#160;': ' ',   # nbsp
-    '&#8209;': '-',  # nbhy
-    '&#8288;': '',   # wj
-}
-
-def get_initials(author):
-    """author is an rfc2629 author element.  Return the author initials,
-    fixed up according to current flavour and policy."""
-    initials = author.attrib.get('initials', '').split()
-    for i in range(len(initials)):
-        initial = initials[i]
-        if not initial.endswith('.'):
-            initials[i] = initial+'.'
-    return ' '.join(initials)
