@@ -266,11 +266,12 @@ class BaseRfcWriter:
         'into languages other than English.'
 
     # Any extra boilerplate
-    boilerplate['iprnotified'] = \
-        'The IETF has been notified of intellectual property rights ' \
-        'claimed in regard to some or all of the specification contained ' \
-        'in this document.  For more information consult the online list ' \
-        'of claimed rights.'
+    # Disabled. See issue #123, http://trac.tools.ietf.org/tools/xml2rfc/trac/ticket/123
+    ## boilerplate['iprnotified'] = \
+    ##     'The IETF has been notified of intellectual property rights ' \
+    ##     'claimed in regard to some or all of the specification contained ' \
+    ##     'in this document.  For more information consult the online list ' \
+    ##     'of claimed rights.'
     
     # Stream approvers
     approvers = {
@@ -305,7 +306,8 @@ class BaseRfcWriter:
         self.pis = xmlrfc.getpis()
 
         # Document counters
-        self.ref_start = 1
+        self.ref_start = 1              # Start of reference counters
+        self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
 
@@ -387,6 +389,30 @@ class BaseRfcWriter:
         self._index.append(item)
         return item
 
+    def _indexRef(self, counter, title=None, anchor=None, toc=False):
+        counter = str(counter)
+        if self.pis['symrefs'] == 'yes':
+            autoName = '[' + (anchor or counter ) +']'
+        else:
+            autoName = '['+counter+']'
+        autoAnchor = 'rfc.ref.' + counter
+        item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
+                       toc=toc)
+        self._index.append(item)
+        return item
+
+    def get_initials(self, author):
+        """author is an rfc2629 author element.  Return the author initials,
+        fixed up according to current flavour and policy."""
+        import re
+        initials = author.attrib.get('initials', '')
+        initials_list = re.split("[. ]+", initials)
+        if self.pis["multiple-initials"] == "no":
+            initials = initials_list[0] + "."
+        else:
+            initials = ". ".join(initials_list) + "."
+        return initials
+
     def _getTocIndex(self):
         return [item for item in self._index if item.toc]
         
@@ -418,7 +444,8 @@ class BaseRfcWriter:
                 date.attrib['year'] = today.strftime('%Y')
                 date.attrib['month'] = today.strftime('%B')
                 date.attrib['day'] = today.strftime('%d')
-        
+            elif year != str(today.year) and not month:
+                xml2rfc.log.warn("Incomplete and out-of date <date/> element: %s" % lxml.etree.tostring(date))
         # Setup the expiration string for drafts as published date + six months
         if self.draft:
             date = self.r.find('front/date')
@@ -474,7 +501,7 @@ class BaseRfcWriter:
             organization = author.find('organization')
             surname = author.attrib.get('surname', '')
             if surname:
-                initials = xml2rfc.utils.get_initials(author)
+                initials = self.get_initials(author)
                 if i == len(authors) - 1 and len(authors) > 1:
                     # Last author is rendered in reverse
                     buf.append('and ' + initials + ' ' + \
@@ -561,27 +588,31 @@ class BaseRfcWriter:
         if authorship == 'yes':
             # Keep track of previous organization and remove if redundant.
             last_org = None
+            last_pos = None
             for author in self.r.findall('front/author'):
                 role = author.attrib.get('role', '')
                 if role == 'editor':
                     role = ', Ed.'
-                initials = xml2rfc.utils.get_initials(author)
+                initials = self.get_initials(author)
                 lines.append(initials + ' ' + author.attrib.\
                              get('surname', '') + role)
                 organization = author.find('organization')
+                org_name = ''
                 if organization is not None:
-                    abbrev = organization.attrib.get('abbrev')
-                    org_result = None
-                    if abbrev:
-                        org_result = abbrev
-                    elif organization.text:
-                        org_result = organization.text
-                    if org_result:
-                        if org_result == last_org:
-                            # Remove redundant organization
-                            lines.remove(last_org)
-                        last_org = org_result
-                        lines.append(org_result)
+                    abbrev = organization.attrib.get("abbrev", None)
+                    if  abbrev != None and abbrev.strip() != '':
+                        org_name = abbrev.strip()
+                    elif organization.text and organization.text.strip() != '':
+                        org_name = organization.text.strip()
+                if org_name == '':
+                    lines.append('')
+                else:
+                    if org_name == last_org:
+                        # Remove redundant organization
+                        del lines[last_pos]
+                    lines.append(org_name)
+                last_org = org_name
+                last_pos = len(lines)-1
 
         date = self.r.find('front/date')
         if date is not None:
@@ -760,7 +791,7 @@ class BaseRfcWriter:
 
         # Set the ending index number so we know where to begin references
         if count_str == '' and appendix == False:
-            self.ref_start = s_count
+            self.refs_start = s_count
 
     def write_status_section(self):
         """ Writes the 'Status of this Memo' section """
@@ -873,7 +904,8 @@ class BaseRfcWriter:
     def _build_index(self):
         self.indexmode = True
         # Reset document counters
-        self.ref_start = 1
+        self.ref_start = 1              # Start of reference counters
+        self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
 
@@ -884,21 +916,27 @@ class BaseRfcWriter:
 
         # References sections
         # Treat references as nested only if there is more than one
-        ref_counter = str(self.ref_start)
+        ref_counter = 0
+        refs_counter = str(self.refs_start)
         references = self.r.findall('back/references')
         # Write root level references header
-        ref_title = self.pis['refparent']
+        refs_title = self.pis['refparent']
         if len(references) == 1:
-            ref_title = references[0].attrib.get('title', ref_title)
+            refs_title = references[0].attrib.get('title', refs_title)
 
-        self._indexReferences(ref_counter, title=ref_title)
+        self._indexReferences(refs_counter, title=refs_title)
 
         if len(references) > 1:
             for i, reference_list in enumerate(references):
-                ref_newcounter = ref_counter + '.' + str(i + 1)
-                ref_title = reference_list.attrib.get('title', self.pis["refparent"])
-                self._indexReferences(ref_newcounter, title=ref_title, \
+                refs_newcounter = refs_counter + '.' + str(i + 1)
+                refs_title = reference_list.attrib.get('title', self.pis["refparent"])
+                self._indexReferences(refs_newcounter, title=refs_title, \
                                       subCounter=i+1, level=2)
+        for reference_list in references:
+            for ref in reference_list:
+                ref_counter += 1
+                title = ref.find("front/title").text
+                self._indexRef(ref_counter, title=title, anchor=ref.attrib["anchor"])
 
         # Appendix sections
         back = self.r.find('back')
@@ -914,20 +952,22 @@ class BaseRfcWriter:
             self._index.append(item)
 
         # Authors addresses section
-        authors = self.r.findall('front/author')
-        autoAnchor = 'rfc.authors'
-        if len(authors) > 1:
-            title = "Authors' Addresses"
-        else:
-            title = "Author's Address"
-        # Add explicitly to index
-        item = _RfcItem(title, autoAnchor, title=title)
-        self._index.append(item)
+        if self.pis["rfcedstyle"] != "yes":
+            authors = self.r.findall('front/author')
+            autoAnchor = 'rfc.authors'
+            if len(authors) > 1:
+                title = "Authors' Addresses"
+            else:
+                title = "Author's Address"
+            # Add explicitly to index
+            item = _RfcItem(title, autoAnchor, title=title)
+            self._index.append(item)
 
     def _build_document(self):
         self.indexmode = False
         # Reset document counters
-        self.ref_start = 1
+        self.ref_start = 1              # Start of reference counters
+        self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
 
@@ -941,6 +981,9 @@ class BaseRfcWriter:
         title = self.r.find('front/title')
         if title is not None:
             docName = self.r.attrib.get('docName', None)
+            rfcnum = self.r.attrib.get('number', None)
+            if (not docName or not docName.strip()) and not rfcnum:
+                xml2rfc.log.warn("No (or empty) 'docName' attribute in the <rfc/> element -- can't insert draft name on first page.")
             self.write_title(title.text, docName, title.sourceline)
 
         # Abstract
@@ -951,8 +994,9 @@ class BaseRfcWriter:
                 self.write_t_rec(t)
 
         # Optional notified boilerplate
-        if self.pis['iprnotified'] == 'yes':
-            self.write_paragraph(BaseRfcWriter.boilerplate['iprnotified'])
+        # Disabled. See issue #123, http://trac.tools.ietf.org/tools/xml2rfc/trac/ticket/123
+        ## if self.pis['iprnotified'] == 'yes':
+        ##    self.write_paragraph(BaseRfcWriter.boilerplate['iprnotified'])
 
         # Optional notes
         for note in self.r.findall('front/note'):
@@ -981,21 +1025,21 @@ class BaseRfcWriter:
 
         # References sections
         # Treat references as nested only if there is more than one
-        ref_counter = str(self.ref_start)
+        refs_counter = str(self.refs_start)
         references = self.r.findall('back/references')
         # Write root level references header
-        ref_title = self.pis['refparent']
+        refs_title = self.pis['refparent']
         if len(references) == 1:
-            ref_title = references[0].attrib.get('title', ref_title)
+            refs_title = references[0].attrib.get('title', refs_title)
 
-        self.write_heading(ref_title, bullet=ref_counter + '.', \
+        self.write_heading(refs_title, bullet=refs_counter + '.', \
                                autoAnchor='rfc.references')
         if len(references) > 1:
             for i, reference_list in enumerate(references):
-                ref_newcounter = ref_counter + '.' + str(i + 1)
-                ref_title = reference_list.attrib.get('title', self.pis['refparent'])
+                refs_newcounter = refs_counter + '.' + str(i + 1)
+                refs_title = reference_list.attrib.get('title', self.pis['refparent'])
                 autoAnchor = 'rfc.references.' + str(i + 1)
-                self.write_heading(ref_title, bullet=ref_newcounter + '.',\
+                self.write_heading(refs_title, bullet=refs_newcounter + '.',\
                                    autoAnchor=autoAnchor, level=2)
                 self.write_reference_list(reference_list)
         elif len(references) == 1:
