@@ -455,13 +455,15 @@ class HtmlRfcWriter(BaseRfcWriter):
 
     def write_reference_list(self, list):
         tbody = E.TBODY()
+        refkeys = []
+        refdict = {}
         for i, reference in enumerate(list.findall('reference')):
             tr = E.TR()
             # Use anchor or num depending on PI
             if self.pis['symrefs'] == 'yes':
-                bullet = reference.attrib.get('anchor', str(i + 1))
+                bullet = reference.attrib.get('anchor', str(i + self.ref_start))
             else:
-                bullet = str(i + 1)
+                bullet = str(i + self.ref_start)
             bullet_td = E.TD(E.B('[' + bullet + ']', id=bullet))
             bullet_td.attrib['class'] = 'reference'
             ref_td = E.TD()
@@ -472,7 +474,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                 organization = author.find('organization')
                 email = author.find('address/email')
                 surname = author.attrib.get('surname')
-                initials = xml2rfc.utils.get_initials(author)
+                initials = self.get_initials(author)
                 if surname is not None:
                     if j == len(authors) - 1 and len(authors) > 1:
                         # Last author, render in reverse
@@ -517,6 +519,10 @@ class HtmlRfcWriter(BaseRfcWriter):
                         seriesInfo.attrib.get('value', '')
                 title_a.tail += seriesInfo.attrib.get('name', '') + ' ' + \
                              seriesInfo.attrib.get('value', '') + ', '
+            if not title_a.attrib.has_key("href"):
+                href = reference.attrib.get("target", None)
+                if href:
+                    title_a.attrib['href'] = href
             date = reference.find('front/date')
             if date is not None:
                 month = date.attrib.get('month', '')
@@ -526,18 +532,23 @@ class HtmlRfcWriter(BaseRfcWriter):
                 title_a.tail += month + year + '.'
             tr.append(bullet_td)
             tr.append(ref_td)
-            tbody.append(tr)
+            refdict[bullet] = tr
+            refkeys.append(bullet)
             # Render annotation as a separate paragraph
             annotation = reference.find('annotation')
             if annotation is not None and annotation.text:
                 ref_td.append(E.P(annotation.text))
-                
+        if self.pis['sortrefs'] == 'yes':
+            refkeys = sorted(refkeys)
+        for key in refkeys:
+            tbody.append(refdict[key])
+        self.ref_start += i + 1                
         # Add to body buffer
         self.buf.append(self._serialize(E.TABLE(tbody)))
 
     def draw_table(self, table, table_num=None):
-        style = 'tt full ' + table.attrib.get('align', 
-                                              self.defaults['table_align'])
+        style = 'tt %s %s' % ( table.attrib.get('style', self.defaults['table_style']),
+                               table.attrib.get('align', self.defaults['table_align']) )
         cellpadding = '3'
         cellspacing = '0'
         htmltable = E.TABLE(cellpadding=cellpadding, cellspacing=cellspacing)
