@@ -76,19 +76,8 @@ class RawTextRfcWriter(BaseRfcWriter):
             to the buffer, with the exception of write_raw() that handles
             preserving of whitespace.
         """
-        if not buf:
+        if buf is None:
             buf = self.buf
-        # We can take advantage of textwrap's initial_indent by using a bullet
-        # parameter and treating it separately.  We still need to indent it.
-        subsequent = ' ' * (indent + sub_indent)
-        if bullet:
-            initial = ' ' * indent + bullet
-            if not sub_indent:
-                # Use bullet length for subsequent indents
-                subsequent = ' ' * len(initial)
-        else:
-            # No bullet, so combine indent and sub_indent
-            initial = subsequent
 
         if leading_blankline:
             if edit and self.pis['editing'] == 'yes':
@@ -97,6 +86,24 @@ class RawTextRfcWriter(BaseRfcWriter):
                 self._lb(buf=buf, text=str('<' + str(self.edit_counter) + '>'))
             else:
                 self._lb(buf=buf)
+
+        # We can take advantage of textwrap's initial_indent by using a bullet
+        # parameter and treating it separately.  We still need to indent it.
+        subsequent = ' ' * (indent + sub_indent)
+        if bullet:
+            initial = ' ' * indent
+            bullet_parts = self.wrapper.wrap(bullet, initial_indent=initial, subsequent_indent=initial,
+                                             fix_doublespace=False, drop_whitespace=False)
+            if len(bullet_parts) > 1:
+                buf.extend(bullet_parts[:-1])
+            initial = bullet_parts[-1]
+            if not sub_indent:
+                # Use bullet length for subsequent indents
+                subsequent = ' ' * len(initial)
+        else:
+            # No bullet, so combine indent and sub_indent
+            initial = subsequent
+
         if string:
             if strip:
                 # Strip initial whitespace
@@ -207,12 +214,19 @@ class RawTextRfcWriter(BaseRfcWriter):
                 t_count += 1
 
         
+    def pre_write_toc(self):
+        return ['', 'Table of Contents', '']
+
+    def post_write_toc(self):
+        return []
+
     def write_toc(self, paging=False):
         """ Write table of contents to a temporary buffer and return """
         if self.toc_marker < 1:
             # Toc is either disabled, or the pointer was messed up
             return ['']
-        tmpbuf = ['', 'Table of Contents', '']
+        tmpbuf = []
+        tmpbuf.extend(self.pre_write_toc())
         # Retrieve toc from the index
         tocindex = self._getTocIndex()
         tocdepth = self.pis['tocdepth']
@@ -257,6 +271,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     # Insert page
                     lines[-1] = lines[-1][:0 - len(pagestr)] + pagestr
                 tmpbuf.extend(lines)
+        tmpbuf.extend(self.post_write_toc())
         return tmpbuf
             
     def write_iref_index(self):
@@ -478,7 +493,7 @@ class RawTextRfcWriter(BaseRfcWriter):
     def write_title(self, title, docName=None, source_line=None):
         """ Write the document title and (optional) name """
         self.write_text(title, leading_blankline=True, align='center', source_line=source_line)
-        if docName is not None:
+        if docName:
             self.write_text(docName, align='center')
 
     def write_heading(self, text, bullet='', autoAnchor=None, anchor=None, \
@@ -665,9 +680,9 @@ class RawTextRfcWriter(BaseRfcWriter):
             annotation = ref.find('annotation')
             # Use anchor or num depending on PI
             if self.pis['symrefs'] == 'yes':
-                bullet = '[' + ref.attrib.get('anchor', str(i + 1)) + ']'
+                bullet = '[' + ref.attrib.get('anchor', str(i + self.ref_start)) + ']'
             else:
-                bullet = '[' + str(i + 1) + ']'
+                bullet = '[' + str(i + self.ref_start) + ']'
             refdict[bullet] = ''.join(refstring)
             refsource[bullet] = ref.sourceline
             refkeys.append(bullet)
@@ -675,7 +690,11 @@ class RawTextRfcWriter(BaseRfcWriter):
             if annotation is not None and annotation.text:
                 # Render annotation as a separate paragraph
                 annotationdict[bullet] = annotation.text
-        if self.pis['sortrefs'] == 'yes':
+        self.ref_start += i + 1
+        # Don't sort if we're doing numbered refs; they are already in
+        # numeric order, and if we sort, they will be sorted alphabetically,
+        # rather than numerically ... ( i.e., [10], [11], ... [19], [1], ... )
+        if self.pis['sortrefs'] == 'yes' and self.pis['symrefs'] == 'yes' :
             refkeys = sorted(refkeys)
         # Hard coded indentation amount
         refindent = 11
@@ -693,6 +712,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         row = 0
         column_aligns = []
         ttcol_width_attrs = []
+        style = table.attrib.get('style', self.defaults['table_style'])        
         for ttcol in table.findall('ttcol'):
             column_aligns.append(ttcol.attrib.get('align',
                                                   self.defaults['ttcol_align']))
@@ -705,12 +725,15 @@ class RawTextRfcWriter(BaseRfcWriter):
         for i, cell in enumerate(table.findall('c')):
             if i % num_columns == 0:
                 # Insert blank row if PI 'compact' is 'no'
-                if self.pis["compact"] == "no" and row > 0:
-                    row += 1
-                    matrix.append(['']*num_columns)
-                    pass
+                if self.pis["compact"] == "no":
+                    if (style == 'none' and row == 0) or (style in ['headers', 'full'] and row != 0):
+                        row += 1
+                        matrix.append(['']*num_columns)
+                        pass
                 row += 1
                 matrix.append([])
+
+
             text = cell.text or ''
             if len(cell) > 0:
                 # <c> has children, render their text and add to line
@@ -721,7 +744,6 @@ class RawTextRfcWriter(BaseRfcWriter):
             matrix[row].append(text)
 
         # Get table style and determine maximum width of table
-        style = table.attrib.get('style', self.defaults['table_style'])
         if style == 'none':
             table_max_chars = self.width - 3
         elif style == 'headers':
@@ -855,12 +877,16 @@ class RawTextRfcWriter(BaseRfcWriter):
                         else:
                             line.append(' ' * (column_widths[col] + 2) + '|')
                 output.append(''.join(line))
-            if i == 0 and style != 'none':
-                # This is the header row, append the header decoration
+            if style in ['headers', 'full']:
+                if i == 0:
+                    # This is the header row, append the header decoration
+                    output.append(''.join(borderstring))
+            if style in ['full']:
+                if i == len(cell_lines)-1:
+                    output.append(''.join(borderstring))
+            if style in ['all']:
                 output.append(''.join(borderstring))
 
-        if not (style == 'headers' or style == 'none'):
-            output.append(''.join(borderstring))
 
         # Finally, write the table to the buffer with proper alignment
         align = table.attrib.get('align', 'center')
@@ -895,9 +921,9 @@ class RawTextRfcWriter(BaseRfcWriter):
         """ Writes the buffer to the specified file """
         # write initial blank lines, not counted against page size...
         if self.draft:
-            file.write(os.linesep*3)
+            file.write("\n"*3)
         else:
-            file.write(os.linesep*6)
+            file.write("\n"*6)
         for line in self.output:
             file.write(line.rstrip(" \t"))
-            file.write(os.linesep)
+            file.write("\n")
